ODA({ is: 'oda-jupyter', imports: '@oda/button',
    template: `
        <style>
            :host{
                @apply --vertical;
                @apply --flex;
                padding: 12px 6px;
                height: calc(100% - 24px);
                opacity: 0;
                transition: opacity ease-out 1s;
            }
        </style>
        <oda-jupyter-divider idx="-1" :focused="!notebook?.cells?.length"></oda-jupyter-divider>
        <div ~for="notebook?.cells" class="vertical no-flex">
            <oda-jupyter-cell :idx="$for.index" :cell="$for.item" :focused="focusedIdx === $for.index" @tap.stop="focusedIdx = (_readOnly ? -1 : $for.index);"></oda-jupyter-cell>
            <oda-jupyter-divider :idx="$for.index" style="margin-top: 4px;"></oda-jupyter-divider>
        </div>
    `,
    $public: {
        $pdp: true,
        iconSize: 24,
        readOnly: false,
        consoleHeight: 160
    },
    $pdp: {
        notebook: null,
        editors: ['Код', 'Текст'],
        focusedIdx: {
            $def: -1,
            set(n) {
                this.editIdx = -1;
            }
        },
        editIdx: -1,
        get _readOnly() {
            return this.notebook?.readOnly || this.readOnly;
        },
        fullCode: ''
    },
    attached() {
        this.async(() => {
            this.style.opacity = 1;
        }, 500)
    },
    $listeners: {
        tap(e) { this.focusedIdx = this.editIdx = -1 }
    }
})

ODA({ is: 'oda-jupyter-divider',
    template: `
        <style>
            :host {
                @apply --vertical;
                height: 8px;
                justify-content: center;
                z-index: 99;
                opacity: 0;
                transition: opacity ease-out .5s;
                position: relative;
                padding: 1px;
            }
            :host([focused]) {
                box-shadow: none !important;
            }
            :host([focused]) {
                opacity: 1;
            }
            :host(:hover) {
                opacity: 1;
            }
            oda-button {
                font-size: 14px;
                margin: 0px 4px;
                @apply --content;
                @apply --border;
                padding: 0px 4px 0px 0px;
                border-radius: 4px;
            }
        </style>
        <div class="horizontal center">
            <div ~if="!_readOnly && idx!==-1" style="width: 100%; position: absolute; top: 2px; height: 1px; border-bottom: 2px solid gray;"></div>
            <oda-button ~if="!_readOnly" :icon-size icon="icons:add" ~for="editors" @tap.stop="addCell($for.item)">{{$for.item}}</oda-button>
        </div>
    `,
    idx: -2,
    addCell(cell_type) {
        let idx = this.idx + 1;
        this.focusedIdx = this.editIdx = -1;
        this.notebook ||= {};
        this.notebook.cells ||= [];
        this.notebook.cells.splice(idx, 0, { cell_type, source: '' });
        this.async(() => this.focusedIdx = idx);
    }
})

ODA({ is: 'oda-jupyter-cell',
    template: `
        <style>
            :host {
                @apply --no-flex;
                position: relative;
                padding: 1px;
                margin: 0 2px;
                transition: opacity ease-out 1s;
            }
        </style>
        <div class="vertical flex main" style="height: 100%; min-height: 22px;">
            <div ~is="this.cellType || 'div'" :idx :cell ~class="{shadow: !_readOnly && focused}"></div>
        </div>
        <oda-jupyter-toolbar :idx ~if="!_readOnly && focused" ~style="{top: '-' + (iconSize - 4) + 'px'}"></oda-jupyter-toolbar>
    `,
    set cell(n) {
        if (n) {
            let type = n.cell_type;
            this.cellType = 'oda-jupyter-' + (type === 'Код' ? 'code' : 'text') + '-editor';
        }
    },
    idx: -2,
    focused: false
})

ODA({ is: 'oda-jupyter-toolbar',
    template: `
        <style>
            :host {
                @apply --horizontal;
                @apply --no-flex;
                @apply --content;
                @apply --shadow;
                position: absolute;
                top: 0;
                right: 8px;
                padding: 1px;
                border-radius: 4px;
                z-index: 100;
            }
        </style>
        <oda-button :disabled="focusedIdx === 0" :icon-size icon="icons:arrow-back:90" @tap="moveCell(-1)"></oda-button>
        <oda-button :disabled="focusedIdx >= notebook?.cells?.length - 1" :icon-size icon="icons:arrow-back:270" @tap="moveCell(1)"></oda-button>
        <span style="width: 8px"></span>
        <oda-button :icon-size icon="icons:delete" @tap="deleteCell"></oda-button>
        <span style="width: 8px"></span>
        <oda-button :icon-size :icon="editIdx===idx?'icons:close':'editor:mode-edit'" @tap="editIdx = editIdx===idx ? -1 : idx"> </oda-button>
    `,
    iconSize: 20,
    cell: null,
    idx: -2,
    moveCell(v) {
        this.editIdx = -1;
        let idx = this.focusedIdx;
        const cells = this.notebook.cells.splice(idx, 1);
        idx = idx + v;
        idx = idx < 0 ? 0 : idx > this.notebook.cells.length ? this.notebook.cells.length : idx;
        this.notebook.cells.splice(idx, 0, cells[0])
        this.async(() => {
            this.focusedIdx = idx;
        })
    },
    deleteCell() {
        if (window.confirm(`Do you really want delete current cell ?`)) {
            this.editIdx = -1;
            this.notebook.cells.splice(this.focusedIdx, 1);
        }
    }
})

ODA({ is: 'oda-jupyter-text-editor', imports: '@oda/simplemde-editor,  @oda/md-viewer',
    template: `
        <style>
            oda-md-viewer::-webkit-scrollbar { width: 0px; height: 0px; }
            :host {
                @apply --horizontal;
                @apply --flex;
            }
            oda-md-viewer {
                opacity: {{opacity}};
                border: none;
                outline: none;
            }
        </style>
        <oda-simplemde-editor ~if="!readOnly && editIdx===idx" style="max-width: 50%; min-width: 50%; padding: 0px; margin: 0px;" @change="_onchange"></oda-simplemde-editor>
        <oda-md-viewer tabindex=0 class="flex" :srcmd="src || _src" :pmargin="'0px'" @dblclick="_setEditMode" @keypress="_keyPress"></oda-md-viewer>
    `,
    set cell(n) {
        this.src = n?.source || '';
    },
    idx: -2,
    src: '',
    _src: 'Чтобы изменить содержимое ячейки, дважды нажмите на нее (или выберите "Ввод")',
    get opacity() {
        return this.src ? 1 : .3;
    },
    _onchange(e) {
        this.cell.source = this.src = e.detail.value;
        this.fire('change', this.cell);
        console.log(this.cell)
    },
    _keyPress(e) {
        if (e.key === 'Enter')
            this._setEditMode();
    },
    _setEditMode() {
        this.editIdx = this.editIdx === this.idx ? -1 : this.idx;
        this.async(() => {
            this.$('oda-simplemde-editor').value = this.src;
        })
    }
})

ODA({ is: 'oda-jupyter-code-editor', imports: '@oda/ace-editor',
    template: `
        <style>
            :host {
                @apply --horizontal;
                @apply --flex;
                outline: 1px solid var(--border-color);
                position: relative;
            }
            oda-icon {
                padding: 4px;
                height: {{iconSize}};
            }
            #icon-close {
                margin-top: {{_top}};
            }
            #icon-close:hover {
                fill: red;
            }
            #j-console { 
                overflow: auto;
                position: relative;
                max-height: {{consoleHeight}}px;
                border-top: 1px solid var(--border-color); 
            }
            #icon-jclose {
                position: absolute;
                bottom: 0px;
            }
            #run {
                width: 100%; 
                border: none;
                min-height: 36px;
                height: 36px;
            }
            #splitter {
                border-top: {{run ? '1px solid var(--border-color)' : 'none'}}; 
            }
        </style>
        <div class="vertical"  style="border-right: 1px solid var(--border-color); padding: 4px 0px">
            <oda-icon :icon-size="iconSize" :icon @pointerover="_icon='av:play-circle-outline'" @tap="_run" @pointerout="_icon=''" style="cursor: pointer; position: sticky; top: 0" :fill="run ? 'green' : 'black'"></oda-icon>
            <oda-icon id="icon-close" ~if="run" :icon-size="iconSize" :icon="_iconClose" @tap="run=false; jConsole = undefined;" style="cursor: pointer; position: sticky; top: 24px"></oda-icon>
        </div>
        <div class="vertical flex">
            <div style="display: block; padding: 2px; font-size: xx-small">{{cell?.mode + ' - ' + (cell?.isODA ? 'isODA' : 'noODA')}}</div>
            <oda-ace-editor :src :mode="cell?.mode || 'javascript'" class="flex" show-gutter="false" max-lines="Infinity" :read-only style="padding: 8px 0" @change="_onchange"></oda-ace-editor>   
            <div id="splitter"></div>
            <iframe id="run" ~if="run" :srcdoc></iframe>
            <div id="j-console" ~if="run && jConsole?.length">
                <div ~for="jConsole" style="padding: 4px;" ~style="jStyle($for.item)">{{$for.item.str}}</div>
            </div>
        </div>
    `,
    jConsole: undefined,
    jStyle(i) {
        let colors = { log: 'gray', info: 'blue', warn: 'orange', error: 'red' };
        return `color: ${colors[i.method]}`;
    },
    src: '',
    set cell(n) {
        this.src = n?.source || '';
        this._setMode(this.src);
    },
    run: false,
    _icon: '',
    _iconClose: 'eva:o-close-circle-outline',
    get _top() {
        return (this.$('#splitter')?.offsetTop) - 36 + 'px';
    },
    srcdoc: '',
    get icon() {
        return this.run ? 'av:play-circle-outline' : this._icon || 'bootstrap:code-square';
    },
    _setMode(src = this.$('oda-ace-editor')?.value || '') {
        this.cell.mode = 'javascript';
        this.cell.isODA = false;
        let oda = src.match(/ODA\b[^(]*\([\s\S]*}\s*?\)/gm);
        if (oda?.length)
            this.cell.isODA = true;
        // let regx = src.match(/<d\b[^>]*>[\s\S]*?<\/script\b[^>]*>/gm);
        let arr = ['</script>', '</html>', '</body>', '</head>', '<link', '<!DOCTYPE html>', '<meta '];
        let regx = new RegExp(arr.join('|'));
        if (regx.test(src))
            this.cell.mode = 'html';
        if (this.cell.mode !== 'html' && !this.cell.isODA) {
            // regx = src.match(/<\b[^>]*>[\s\S]*?<\/\b[^>]*>/gmi);
            regx = src.match(/<\b[^>]*>[\s\S]*?/gm);
            this.cell.mode = src.match(regx)?.length ? 'html' : 'javascript';
        }
        // console.log(this.cell.mode)
    },
    getFullCode() {
        this.fullCode = '';
        let html = '', script = '', isODA = false;
        this.notebook.cells.map(i => {
            // if (i.mode === 'html')
            //     html += i.source + `
            // `;
            if (i.mode === 'javascript')
                script += i.source + `

`;
                isODA ||= i.isODA;
        })
        let odaImport = isODA ? `import '../../oda.js';` : ``;
        script = 
`
${this.cell.mode === 'html' ? this.cell.source : ''}

<script type="module">
${odaImport}

${script}
</script>
`;
        // console.log(script)
        return script;
    },
    _onchange(e) {
        this._top = undefined;
        this.cell.source = e.detail.value;
        this.fire('change', this.cell);
        this._setMode();
    },
    _run() {
        this._top = undefined;
        this._iconClose = 'spinners:180-ring-with-bg';
        this.srcdoc = '';
        this.run = true;
        this.async(() => {
            const iframe = this.$('iframe');
            this.srcdoc = `
<style>
    html, body {
        margin: 0;
        padding: 0;
        position: relative;
        font-family: monospace;
        font-size: 18px;
    }
    * *, *:before, *:after {  
        box-sizing: border-box;
    }
</style>
<script async>
    window._jConsole = [];
    var takeOverConsole = () => {
        var console = window.console;
        if (!console) return;
        window.print = (e) => console.log(e);
        window.log = (e) => console.log(e);
        window.info = (e) => console.info(e);
        window.warn = (e) => console.warn(e);
        window.error = (e) => console.error(e);
        var intercept = (method) => {
            var original = console[method];
            console[method] = function() {
                var message = arguments;
                if (original.apply) {
                    // Do this for normal browsers
                    original.apply(console, arguments);
                } else {
                    // Do this for IE
                    message = Array.prototype.slice.apply(arguments).join(' ');
                    original(message);
                }
                if (window.jConsole) {
                    window.jConsole.unshift({ method, str: Array.prototype.slice.apply(message).join(' ') });
                } else {
                    window._jConsole.unshift({ method, str: Array.prototype.slice.apply(message).join(' ') });
                }
            }
        }
        ['log', 'info', 'warn', 'error'].forEach(i => intercept(i));
    }
    takeOverConsole();
</script>
            ` + this.getFullCode();
            this.async(() => {
                this.jConsole = [...iframe.contentWindow._jConsole];
                iframe.contentWindow.jConsole = this.jConsole;
                iframe.style.height = iframe.contentDocument.body.scrollHeight + 'px';
                iframe.style.opacity = 1;
                this._iconClose = 'eva:o-close-circle-outline';
            }, 300)
        }, 100)
    }
})
